# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# listener_dag_function.py
from __future__ import annotations

import json
import os

from pendulum import datetime

from airflow import DAG

# Connections needed for this example dag to finish
from airflow.models import Connection
from airflow.providers.apache.kafka.operators.produce import ProduceToTopicOperator
from airflow.providers.apache.kafka.sensors.kafka import AwaitMessageTriggerFunctionSensor

# This is just for setting up connections in the demo - you should use standard
# methods for setting these connections in production
from airflow.providers.standard.operators.python import PythonOperator


def load_connections():
    conns = [
        Connection(
            conn_id="fizz_buzz_1",
            conn_type="kafka",
            extra=json.dumps({"socket.timeout.ms": 10, "bootstrap.servers": "broker:29092"}),
        ),
        Connection(
            conn_id="fizz_buzz_2",
            conn_type="kafka",
            extra=json.dumps(
                {
                    "bootstrap.servers": "broker:29092",
                    "group.id": "fizz_buzz",
                    "enable.auto.commit": False,
                    "auto.offset.reset": "beginning",
                }
            ),
        ),
    ]

    for c in conns:
        envvar = f"AIRFLOW_CONN_{c.conn_id.upper()}"
        os.environ[envvar] = c.get_uri()


def _producer_function():
    for i in range(50):
        yield (json.dumps(i), json.dumps(i + 1))


with DAG(
    dag_id="fizzbuzz-load-topic",
    description="Load Data to fizz_buzz topic",
    start_date=datetime(2022, 11, 1),
    schedule=None,
    catchup=False,
    tags=["fizz-buzz"],
) as dag:
    t0 = PythonOperator(task_id="load_connections", python_callable=load_connections)

    t1 = ProduceToTopicOperator(
        kafka_config_id="fizz_buzz_1",
        task_id="produce_to_topic",
        topic="fizz_buzz",
        producer_function=_producer_function,
    )

with DAG(
    dag_id="fizzbuzz-listener-dag",
    description="listen for messages with mod 3 and mod 5 are zero",
    start_date=datetime(2022, 11, 1),
    catchup=False,
    tags=["fizz", "buzz"],
):

    def await_function(message):
        val = json.loads(message.value())
        print(f"Value in message is {val}")
        if val % 3 == 0:
            return val
        if val % 5 == 0:
            return val

    def wait_for_event(message, **context):
        if message % 15 == 0:
            return f"encountered {message}!"
        if message % 3 == 0:
            print(f"encountered {message} FIZZ !")
        if message % 5 == 0:
            print(f"encountered {message} BUZZ !")

    # [START howto_sensor_await_message_trigger_function]
    listen_for_message = AwaitMessageTriggerFunctionSensor(
        kafka_config_id="fizz_buzz_2",
        task_id="listen_for_message",
        topics=["fizz_buzz"],
        apply_function="example_dag_event_listener.await_function",
        event_triggered_function=wait_for_event,
    )
    # [END howto_sensor_await_message_trigger_function]

    t0 >> t1


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
import logging
from typing import Any

import pytest

from airflow.models import Connection
from airflow.providers.apache.kafka.operators.produce import ProduceToTopicOperator

log = logging.getLogger(__name__)


def _simple_producer(key, value) -> list[tuple[Any, Any]]:
    """simple_producer A function that returns the key,value passed
    in for production via "KafkaProducerOperator"

    :param key: the key for the message
    :param value: the value for the message
    :return: The Key / Value pair for production via the operator
    :rtype: list[tuple[Any, Any]]
    """
    return [(key, value)]


class TestProduceToTopic:
    """
    Test ConsumeFromTopic
    """

    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(
                    {
                        "socket.timeout.ms": 10,
                        "message.timeout.ms": 10,
                        "bootstrap.servers": "localhost:9092",
                        "group.id": "test_group",
                    }
                ),
            )
        )

    def test_operator_string(self):
        operator = ProduceToTopicOperator(
            kafka_config_id="kafka_d",
            topic="test_1",
            producer_function="unit.apache.kafka.operators.test_produce._simple_producer",
            producer_function_args=(b"test", b"test"),
            task_id="test",
            synchronous=False,
        )

        operator.execute(context={})

    def test_operator_callable(self):
        operator = ProduceToTopicOperator(
            kafka_config_id="kafka_d",
            topic="test_1",
            producer_function=_simple_producer,
            producer_function_args=(b"test", b"test"),
            task_id="test",
            synchronous=False,
        )

        operator.execute(context={})



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
import logging

import pytest
from confluent_kafka import Consumer

from airflow.models.connection import Connection
from airflow.providers.apache.kafka.operators.produce import ProduceToTopicOperator

log = logging.getLogger(__name__)


def _producer_function():
    for i in range(20):
        yield (json.dumps(i), json.dumps(i + 1))


@pytest.fixture(autouse=True)
def kafka_connections(create_connection_without_db):
    """Create Kafka producer connections for testing purpose."""
    connections = [
        Connection(
            conn_id="kafka_default_test_1",
            uri="kafka://broker:29092?socket.timeout.ms=10&message.timeout.ms=10&group.id=operator.producer.test.integration.test_1",
        ),
        Connection(
            conn_id="kafka_default_test_2",
            uri="kafka://broker:29092?socket.timeout.ms=10&message.timeout.ms=10&group.id=operator.producer.test.integration.test_2",
        ),
    ]

    for conn in connections:
        create_connection_without_db(conn)


@pytest.mark.integration("kafka")
class TestProduceToTopic:
    """
    test ProduceToTopicOperator
    """

    def test_producer_operator_test_1(self):
        GROUP = "operator.producer.test.integration.test_1"
        TOPIC = "operator.producer.test.integration.test_1"

        t = ProduceToTopicOperator(
            kafka_config_id="kafka_default",
            task_id="produce_to_topic",
            topic=TOPIC,
            producer_function="integration.apache.kafka.operators.test_produce._producer_function",
        )

        t.execute(context={})

        config = {
            "bootstrap.servers": "broker:29092",
            "group.id": GROUP,
            "enable.auto.commit": False,
            "auto.offset.reset": "beginning",
        }

        c = Consumer(config)
        c.subscribe([TOPIC])
        msg = c.consume()

        assert msg[0].key() == b"0"
        assert msg[0].value() == b"1"

    def test_producer_operator_test_2(self):
        GROUP = "operator.producer.test.integration.test_2"
        TOPIC = "operator.producer.test.integration.test_2"

        t = ProduceToTopicOperator(
            kafka_config_id="kafka_default",
            task_id="produce_to_topic",
            topic=TOPIC,
            producer_function=_producer_function,
        )

        t.execute(context={})

        config = {
            "bootstrap.servers": "broker:29092",
            "group.id": GROUP,
            "enable.auto.commit": False,
            "auto.offset.reset": "beginning",
        }

        c = Consumer(config)
        c.subscribe([TOPIC])
        msg = c.consume()

        assert msg[0].key() == b"0"
        assert msg[0].value() == b"1"



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

import json
import logging

import pytest

from airflow.models import Connection
from airflow.providers.apache.kafka.sensors.kafka import AwaitMessageSensor, AwaitMessageTriggerFunctionSensor
from airflow.providers.common.compat.sdk import TaskDeferred

log = logging.getLogger(__name__)


def _return_true(message):
    return True


class TestSensors:
    """
    Test Sensors
    """

    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(
                    {"socket.timeout.ms": 10, "bootstrap.servers": "localhost:9092", "group.id": "test_group"}
                ),
            )
        )

    def test_await_message_good(self):
        sensor = AwaitMessageSensor(
            kafka_config_id="kafka_d", topics=["test"], task_id="test", apply_function=_return_true
        )

        # execute marks the task as deferred
        with pytest.raises(TaskDeferred):
            sensor.execute(context={})

    def test_await_execute_complete(self):
        sensor = AwaitMessageSensor(
            kafka_config_id="kafka_d", topics=["test"], task_id="test", apply_function=_return_true
        )

        assert sensor.execute_complete(context={}, event="test") == "test"

    def test_await_message_trigger_event(self):
        sensor = AwaitMessageTriggerFunctionSensor(
            kafka_config_id="kafka_d",
            topics=["test"],
            task_id="test",
            apply_function=_return_true,
            event_triggered_function=_return_true,
        )

        # task should immediately come out of deferred
        with pytest.raises(TaskDeferred):
            sensor.execute(context={})

    def test_await_message_trigger_event_execute_complete(self):
        sensor = AwaitMessageTriggerFunctionSensor(
            kafka_config_id="kafka_d",
            topics=["test"],
            task_id="test",
            apply_function=_return_true,
            event_triggered_function=_return_true,
        )

        # task should immediately come out of deferred
        with pytest.raises(TaskDeferred):
            sensor.execute_complete(context={})

    def test_await_message_with_timeout_parameter(self):
        """Test that AwaitMessageSensor accepts timeout parameter."""
        sensor = AwaitMessageSensor(
            kafka_config_id="kafka_d",
            topics=["test"],
            task_id="test",
            apply_function=_return_true,
            timeout=600,  # This should now work without errors
        )

        assert sensor.timeout == 600

    def test_await_message_with_soft_fail_parameter(self):
        """Test that AwaitMessageSensor accepts soft_fail parameter."""
        sensor = AwaitMessageSensor(
            kafka_config_id="kafka_d",
            topics=["test"],
            task_id="test",
            apply_function=_return_true,
            soft_fail=True,  # This should now work without errors
        )

        assert sensor.soft_fail is True

    def test_await_message_trigger_function_with_timeout_parameter(self):
        """Test that AwaitMessageTriggerFunctionSensor accepts timeout parameter."""
        sensor = AwaitMessageTriggerFunctionSensor(
            kafka_config_id="kafka_d",
            topics=["test"],
            task_id="test",
            apply_function=_return_true,
            event_triggered_function=_return_true,
            timeout=600,
        )

        assert sensor.timeout == 600

    def test_await_message_trigger_function_with_soft_fail_parameter(self):
        """Test that AwaitMessageTriggerFunctionSensor accepts soft_fail parameter."""
        sensor = AwaitMessageTriggerFunctionSensor(
            kafka_config_id="kafka_d",
            topics=["test"],
            task_id="test",
            apply_function=_return_true,
            event_triggered_function=_return_true,
            soft_fail=True,
        )

        assert sensor.soft_fail is True



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json

import pytest

from airflow.models import Connection
from airflow.providers.apache.kafka.hooks.client import KafkaAdminClientHook

client_config = {"socket.timeout.ms": 1000, "bootstrap.servers": "broker:29092", "group.id": "my-group"}


@pytest.mark.integration("kafka")
class TestKafkaAdminClientHook:
    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(client_config),
            )
        )

    def test_hook(self):
        """test the creation of topics"""

        # Standard Init
        hook = KafkaAdminClientHook(kafka_config_id="kafka_d")
        hook.create_topic(topics=[("test_1", 1, 1), ("test_2", 1, 1)])

        kadmin = hook.get_conn
        t = kadmin.list_topics(timeout=10).topics
        assert t.get("test_2")
        hook.delete_topic(topics=["test_1", "test_2"])
